/*
 * coax
 * https://github.com/jchris/coax
 *
 * Copyright (c) 2013 Chris Anderson
 * Licensed under the Apache license.
 */
var pax = require("pax"),
  hoax = require("hoax");

var coaxPax = pax();

coaxPax.extend("getQuery", function(params) {
  params = JSON.parse(JSON.stringify(params));
  var key, keys = ["key", "startkey", "endkey", "start_key", "end_key"];
  for (var i = 0; i < keys.length; i++) {
    key = keys[i];
    if (params[key]) {
      params[key] = JSON.stringify(params[key]);
    }
  }
  return params;
});

var Coax = module.exports = hoax.makeHoax(coaxPax());

Coax.extend("changes", function(opts, cb) {
  if (typeof opts === "function") {
    cb = opts;
    opts = {};
  }
  var self = this;
  opts = opts || {};


  if (opts.feed == "continuous") {
    var listener = self(["_changes", opts], function(err, ok) {
      if (err && err.code == "ETIMEDOUT") {
        return self.changes(opts, cb); // TODO retry limit?
      } else if (err) {
        return cb(err);
      }
    });
    listener.on("data", function(data){
      var sep = "\n";

      // re-emit chunked json data
      eom = data.toString().indexOf(sep)
      msg = data.toString().substring(0, eom)
      remaining = data.toString().substring(eom + 1, data.length)
      if (remaining.length > 0){
        // console.log(data.toString())
        listener.emit("data", remaining)
      }

      var json = JSON.parse(msg);
      cb(false, json)
    })
    return listener;
  } else {
    opts.feed = "longpoll";
    // opts.since = opts.since || 0;
    // console.log("change opts "+JSON.stringify(opts));
    return self(["_changes", opts], function(err, ok) {
      if (err && err.code == "ETIMEDOUT") {
        return self.changes(opts, cb); // TODO retry limit?
      } else if (err) {
        return cb(err);
      }
      // console.log("changes", ok)
      ok.results.forEach(function(row){
        cb(null, row);
      });
      opts.since = ok.last_seq;
      self.changes(opts, cb);
    });
  }
});

Coax.extend("forceSave", function(doc, cb) {
  var api = this(doc._id);
  // console.log("forceSave "+api.pax);
  api.get(function(err, old) {
    if (err && err.error !== "not_found") {
      return cb(err);
    }
    if (!err) {
      doc._rev = old._rev;
    }
    // console.log("forceSave put", api.pax, doc._rev)
    api.put(doc, cb);
  });
});


Coax.extend("channels", function(channels, opts) {
  var self = this;
  var opts = opts || {};

  opts.filter = "sync_gateway/bychannel";
  opts.feed = "continuous"
  opts.channels = channels.join(',')

  // console.log(self.pax.toString())
  var x = function(){};
  x.request = true;
  var changes = self(['_changes', opts], x);
  changes.on("data", function(data) {
    var json;
    try{
      var json = JSON.parse(data.toString())
    }catch(e){
      console.log("not json", data.toString())
    }
    if (json) {
      changes.emit("json", json)
    }
  })
  return changes;
});
